// Font.cpp

#include "Font.h"

/*
 *====================================================================
 *					DrawFont()
 *
 * desc: Draw's part's of the font file onto the screen to display a
 *       string of letters
 * @param: Two SDL_Surfaces, one is the main game screen surface, the
 *         other is the image font file. x and y are used to assign
 *         it's coordinates, x2 and y2 do the same. W is it's width
 *         and h is it's height
 *====================================================================
 */
void Font::DrawFont(SDL_Surface *screen, SDL_Surface *img, int x,
										int y, int w, int h, int x2, int y2)
{
  SDL_Rect dest;
  dest.x = x;
  dest.y = y;
  SDL_Rect src;
  src.x = x2;
  src.y = y2;
  src.w = w;
  src.h = h;
  SDL_BlitSurface(img, &src, screen, &dest);
}

/*
 *====================================================================
 *          initFont()
 *
 * desc: Loads the font(s) that will be used in the program to display
 *       text
 * @param: A char pointer named fontdir where the font is located, and
 *         four floating point variables that will assign the red, blue,
 *         green and alpha value for the masks
 *====================================================================
 */
SDLFont* Font::initFont(char *fontdir, float r, float g, float b, float a)
{
  // some variables
  SDLFont *tempFont;               // a temporary font
  FILE *fp;                        // file pointer - used when reading files
  char tempString[100];            // temporary string
  unsigned char tmp;               // temporary unsigned char
  int width;                       // the width of the font
  SDL_Surface *tempSurface;        // temporary surface

  // find out about the size of a font from the ini file
  sprintf(tempString,"%s/%s",fontdir,"font.ini");
  fp = fopen(tempString, "rb");
  if( fp == NULL )
  {
    return 0;
  }
  fscanf(fp, "%d", &width);
  fclose(fp);

  // let's create our font structure now
  tempFont = new SDLFont;
  tempFont->data = new unsigned char[width*width*4];
  tempFont->width = width;
  tempFont->charWidth = width/16;

  // open the font raw data file and read everything in
  sprintf(tempString,"%s/%s",fontdir,"font.raw");
  fp = fopen(tempString, "rb");
  if( fp != NULL )
  {
    for(int i=0;i<width*width;i++)
    {
      tmp = fgetc(fp);
      tempFont->data[i*4] = (unsigned char)255*(unsigned char)r;
      tempFont->data[i*4+1] = (unsigned char)255*(unsigned char)g;
      tempFont->data[i*4+2] = (unsigned char)255*(unsigned char)b;
      tempFont->data[i*4+3] = (unsigned char)(((float)tmp)*a);
    }
  } else {
    return 0;
  }
  fclose(fp);
  // now let's create a SDL surface for the font
  Uint32 rmask,gmask,bmask,amask;
  #if SDL_BYTEORDER == SDL_BIG_ENDIAN
  rmask = 0xff000000;
  gmask = 0x00ff0000;
  bmask = 0x0000ff00;
  amask = 0x000000ff;
  #else
  rmask = 0x000000ff;
  gmask = 0x0000ff00;
  bmask = 0x00ff0000;
  amask = 0xff000000;
  #endif
  tempSurface = SDL_CreateRGBSurfaceFrom(tempFont->data, width, width,
                              32, width*4, rmask, gmask, bmask, amask);
  tempFont->font = SDL_DisplayFormatAlpha(tempSurface);
  SDL_FreeSurface(tempSurface);

  // let's create a variable to hold all the widths of the font
  tempFont->widths = new int[256];

  // now read in the information about the width of each character
  sprintf(tempString,"%s/%s",fontdir,"font.dat");
  fp = fopen(tempString, "rb");
  if( fp != NULL )
  {
    for(int i=0;i<256;i++)
    {
      tmp = fgetc(fp);
      tempFont->widths[i]=tmp;
    }
  }
  fclose(fp);

  /// return the new font
  return tempFont;
}

/*
 *====================================================================
 *					drawString()
 *
 * desc: Draw's the requested string onto the screen
 * @param: An SDL_Surface that the string will be printed to, an
 *         SDL_Font structure that is used to hold the font's data,
 *         it's x and y coordinates, the string to be printed and
 *         other data if needed
 *====================================================================
 */
void Font::drawString(SDL_Surface *screen, SDLFont *font, int x, int y, char *str, ...)
{
  char string[1024];                  // Temporary string

  va_list ap;                         // Pointer To List Of Arguments
  va_start(ap, str);                  // Parses The String For Variables
  vsprintf(string, str, ap);          // And Converts Symbols To Actual Numbers
  va_end(ap);                         // Results Are Stored In Text

  int len=strlen(string);             // Get the number of chars in the string
  int xx=0;														// This will hold the place where to draw the next char
  for(int i=0;i<len;i++)              // Loop through all the chars in the string
  {
    DrawFont( screen, font->font, xx+x, y, font->widths[string[i]]+2, font->charWidth,
      (string[i]%16*font->charWidth)+((font->charWidth/2)-(font->widths[string[i]])/2),
      (((int)string[i]/16)*font->charWidth)
						);

    xx+=font->widths[string[i]];
  }
}

/*
 *====================================================================
 *					stringWidth()
 *
 * desc: Get's the length of a string that will be printed to the
 *       string
 * @param: An SDLFont structure that will be used, the string that
 *         will be printed, and some other data
 *====================================================================
 */
int Font::stringWidth(SDLFont *font, char *str, ...)
{
  char string[1024];                  // Temporary string

  va_list ap;                         // Pointer To List Of Arguments
  va_start(ap, str);                  // Parses The String For Variables
  vsprintf(string, str, ap);          // And Converts Symbols To Actual Numbers
  va_end(ap);                         // Results Are Stored In Text

  // Now we just count up the width of all the characters
  int xx=0;
  int len=strlen(string);
  for(int i=0;i<len;i++)
  {
    // Add their widths together
    xx+=font->widths[string[i]];
  }

  // and then return the sum
  return xx;
}

/*
 *====================================================================
 *					freeFont()
 *
 * desc: Free's up all the allocated space used by the font - it's
 *       only polite to clean up the messes you make :)
 * @param: An SDLFont structure to first clear out and then delete
 *====================================================================
 */
void Font::freeFont(SDLFont *font)
{
	delete[] font->widths;
	delete[] font->data;
	SDL_FreeSurface(font->font);
	delete font;
}